# Copyright: (c) 2022 Jordan Borean (@jborean93) <jborean93@gmail.com>
# MIT License (see LICENSE or https://opensource.org/licenses/MIT)

from krb5._context import Context

def enctype_to_string(
    context: Context,
    enctype: int,
) -> str:
    """Convert an encryption type to a string.

    Converts the encryption type identifier to the string name representation.

    Note:
        This API is marked as public but should not be called directly in MIT.

    Note:
        The Heimdal and MIT implementation return quite different values. It is
        recommended to use :meth:`enctype_to_name` if available on MIT to get
        a common value back.

    Args:
        context: Krb5 context.
        enctype: The encryption type identifier to convert.

    Returns:
        str: The encryption type name.
    """

def string_to_enctype(
    context: Context,
    string: str,
) -> int:
    """Convert string to encryption type.

    Converts a string to an encryption type integer.

    Args:
        context: Krb5 context.
        string: The string to convert from.

    Returns:
        int: The encryption type.
    """



# Copyright: (c) 2021 Jordan Borean (@jborean93) <jborean93@gmail.com>
# MIT License (see LICENSE or https://opensource.org/licenses/MIT)

import enum
import typing

from krb5._context import Context

class PrincipalParseFlags(enum.IntEnum):
    """Flags used to control :meth:`parse_name_flags`."""

    none = ...  #: No parse flags set
    no_realm = ...  #: Error if realm is present
    require_realm = ...  #: Error if realm is not present
    enterprise = ...  #: Create single-component enterprise principal
    ignore_realm = ...  #: Ignore realm if present

class PrincipalUnparseFlags(enum.IntEnum):
    """Flags used to control :meth:`unparse_name_flags`."""

    none = ...  #: No unparse flags set
    short = ...  #: Omit realm if it is the local realm
    no_realm = ...  #: Omit realm always
    display = ...  #: Don't escape special characters

class NameType(enum.IntEnum):
    """A kerberos principal name type"""

    unknown = 0  #: Name type not known
    principal = 1  #: Just the name of the principal as in DCE, or for users
    srv_inst = 2  #: Service and other unique instance (krbtgt)
    srv_hst = 3  #: Service with host name as instance (telnet, rcommands)
    srv_xhst = 4  #: Service with host as remaining components
    uid = 5  #: Unique ID
    x500_principal = 6  #: Encoded X.509 Distinguished name [RFC2253]
    smtp_name = 7  #: Name in form of SMTP email name (e.g., user@example.com)
    enterprise_principal = 10  #: Enterprise name; may be mapped to principal name
    wellknown = 11  #: Well-known (special) principal
    ms_principal = -128  #: Windows 2000 UPN and SID
    ms_principal_and_id = -129  #: NT 4 style name
    ent_principal_and_id = -130  #: NT 4 style name and SID

class Principal:
    """Kerberos Principal object.

    This class represents a Kerberos principal.

    Args:
        context: Krb5 context.
    """

    def __copy__(self) -> "Principal":
        """Create a copy of the principal object."""

    @property
    def addr(self) -> typing.Optional[int]:
        """The raw krb5_principal pointer address of this credential cache."""

    @property
    def name(self) -> typing.Optional[bytes]:
        """The name of the principal."""

    @property
    def realm(self) -> bytes:
        """The realm of the principal."""

    @property
    def components(self) -> typing.List[bytes]:
        """The list of components of the principal."""

    @property
    def type(self) -> NameType:
        """The name type of the principal."""

    @type.setter
    def type(self, value: NameType) -> None:
        pass

def copy_principal(
    context: Context,
    principal: Principal,
) -> Principal:
    """Copy a principal.

    Creates a copy of the principal specified.

    Args:
        context: Krb5 context.
        principal: The principal to copy.

    Returns:
        Principal: The copy of the principal.
    """

def parse_name_flags(
    context: Context,
    name: bytes,
    flags: typing.Union[int, PrincipalParseFlags] = PrincipalParseFlags.none,
) -> Principal:
    """Create a Kerberos principal.

    Convert a string principal name to a Kerberos principal object.

    Args:
        context: Krb5 context.
        name: The principal name to parse.
        flags: Optional flags to control how the string is parsed.

    Returns:
        Principal: The Kerberos principal parsed from the string.
    """

def unparse_name_flags(
    context: Context,
    principal: Principal,
    flags: typing.Union[int, PrincipalUnparseFlags] = PrincipalUnparseFlags.none,
) -> bytes:
    """Get the Kerberos principal name.

    Converts a Kerberos principal to a string representation.

    args:
        context: Krb5 context.
        principal: The principal to convert from.
        flags: Optional flags to control how the string is generated.

    Returns:
        bytes: The principal as a byte string.
    """

def build_principal(
    context: Context,
    realm: bytes,
    components: typing.Iterable[bytes],
) -> Principal:
    """Build a principal name.

    Create a Kerberos principal from a realm name and a list of components.

    args:
        context: Krb5 context.
        realm: The realm name.
        components: The list of components.

    Returns:
        bytes: The principal as a byte string.
    """



# Copyright: (c) 2021 Jordan Borean (@jborean93) <jborean93@gmail.com>
# MIT License (see LICENSE or https://opensource.org/licenses/MIT)

import typing

from krb5._context import Context

def init_secure_context() -> Context:
    """Create a secure krb5 library context.

    Create a context structure, using only system configuration files. All
    information passed through environment variables are ignored.

    Returns:
        Context: The opened krb5 library context.
    """

def get_time_offsets(
    context: Context,
) -> typing.Tuple[int, int]:
    """Return the time offset of the specified context.

    Returns the time offset of the specified context.

    Args:
        context: Krb5 context.

    Returns:
        The seconds and microseconds of the time offset.
    """



# Copyright: (c) 2021 Jordan Borean (@jborean93) <jborean93@gmail.com>
# MIT License (see LICENSE or https://opensource.org/licenses/MIT)

from krb5._context import Context
from krb5._kt import KeyTab

def kt_get_full_name(
    context: Context,
    keytab: KeyTab,
) -> bytes:
    """Retrieve the full name of the keytab.

    Retrieves the full name of the keytab in the form ``type:residual``.

    Args:
        Context: Krb5 context.
        keytab: The keytab to query.

    Returns:
        bytes: The fullname of the keytab.
    """



from krb5._context import Context

def chpw_message(context: Context, server_response: bytes) -> bytes:
    """This function processes the byte sequence returned as the
    `server_response` by :meth:`set_password()` and
    :meth:`set_password_using_ccache()` functions, and returns a human readable
    byte string.

    Note that `gettext` library is used to translate the strings according
    to locale settings. For the list of existing translations, pls. refer
    to MIT krb5 source code. Not all translations may be available on your
    system. Caller is responsible for decoding the string according to
    locale settings.

    Args:
        context: Krb5 context.
        server_response: The `server_response` bytes received from the KDC.

    Returns:
        bytes: The human readable bytes string.
    """
